#include "main.h"

CGU_Settings CGU0_PARAM_LIST_DEFAULT;
DMC_PARAM DMC_Param_List;

int main(void)
{
	int failure=0; int i;

	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=LPDDR_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=2;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize CGU - PLLCLK = 400 MHz, CCLK = 400 MHz, SYCLK = 200 MHz, DCLK = 200 MHz, SCLK0 = SCLK1 = 100 MHz
	CGUInit();

	//Initialize DMC0 now
	DMCInit(DMC_Param_List);

	DMC_Param_List.dmc_no=1;
	DMC_Param_List.dmc_data_calib_add_value=DMC1_DATA_CALIB_ADD;

	//Initialize DMC1 now
	DMCInit(DMC_Param_List);

	//Print DMC register contents for both the DMCs
	Print_DMC_Register_Contents(0);
	Print_DMC_Register_Contents(1);

	printf("\nTesting for DMC0...\n");
	failure=Memory_Sweep_Test(DMC0_START_ADDRESS, 0x100000);

	printf("\nTesting for DMC1...\n");
	failure=Memory_Sweep_Test(DMC1_START_ADDRESS, 0x100000);

	if(failure==0)
		printf("\nTest passed...\n");
	else
		printf("\nTest failed...\n");

	return 0;
}

void CGUInit(void)
{
	int i;

	//PLLCLK = 400 MHz, CCLK = 400 MHz, SYCLK = 200 MHz, DCLK = 200 MHz, SCLK0 = SCLK1 = 100 MHz
	CGU0_PARAM_LIST_DEFAULT.ctl_MSEL=16;
	CGU0_PARAM_LIST_DEFAULT.ctl_DF=0;
	CGU0_PARAM_LIST_DEFAULT.div_CSEL=1;
	CGU0_PARAM_LIST_DEFAULT.div_SYSSEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S0SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S1SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_DSEL=2;

	//Set DMCx_PHY_CTL0.RESETDLL bit.
	*pREG_DMC0_PHY_CTL0=0x800;
	*pREG_DMC1_PHY_CTL0=0x800;

	//Initialize CGU to change the DCLK frequency
	CGU_Init(0, 0, &CGU0_PARAM_LIST_DEFAULT,  0);

	//Clear DMCx_PHY_CTL0.RESETDLL bit
	*pREG_DMC0_PHY_CTL0=0x000;
	*pREG_DMC1_PHY_CTL0=0x000;

	//Wait for 9000 DCLK cycles for the DLL to lock
	for(i=0;i<9000*DMC_Param_List.cclk_dclk_ratio;i++);
}

